@@ -7,14 +7,30 @@ use constant SYSCTL_NOT_EXISTS  =>
     qr{second \s level \s name .+? in .+? is \s invalid}xms,
     qr{name                    .+? in .+? is \s unknown}xms,
 ;
-use constant RE_SYSCTL_SPLIT => qr{\n+}xms;
-use constant RE_SYSCTL_ROW   => qr{:(?:\s)+?}xms;
+use constant RE_SYSCTL_SPLIT   => qr{\n+}xms;
+use constant RE_SYSCTL_ROW     => qr{:(?:\s)+?}xms;
+use constant RE_OLD_SYSCTL_ROW => qr{(?:\s)+?=(?:\s)+?}xms;
 
 use Capture::Tiny qw( capture );
 use Carp          qw( croak   );
 
-our $VERSION = '0.793';
-our @EXPORT  = qw( fsysctl nsysctl sw_vers system_profiler );
+our $VERSION = '0.794';
+our @EXPORT  = qw(
+    fsysctl
+    nsysctl
+    plist
+    sw_vers
+    system_profiler
+);
+
+sub plist {
+    my $thing = shift;
+    my $raw   = $thing !~ m{\n}xms && -e $thing
+              ? __PACKAGE__->slurp( $thing )
+              : $thing;
+    require Mac::PropertyList;
+    return Mac::PropertyList::parse_plist( $raw )->as_perl;
+}
 
 sub system_profiler {
     # SPSoftwareDataType -> os version. user
@@ -25,8 +41,7 @@ sub system_profiler {
         system system_profiler => '-xml', (@types ? @types : ())
     };
 
-    require Mac::PropertyList;
-    my $raw = Mac::PropertyList::parse_plist( $out )->as_perl;
+    my $raw = plist( $out );
 
     my %rv;
     foreach my $e ( @{ $raw } ) {
@@ -69,14 +84,7 @@ sub _sysctl {
         foreach my $row ( split RE_SYSCTL_SPLIT, $out ) {
             chomp $row;
             next if ! $row;
-            my($name, $value) = split RE_SYSCTL_ROW, $row, 2;
-            if ( ! $value && $value ne '0' ) {
-                croak sprintf q(Can't happen: No value in output for property )
-                            . q('%s' inside row '%s' collected from key '%s'),
-                                $name || q([no name]),
-                                $row,
-                                $key;
-            }
+            my($name, $value) = _parse_sysctl_row( $row, $key );
             $rv{ $name } = $value;
         }
     }
@@ -92,6 +100,24 @@ sub _sysctl {
     };
 }
 
+sub _parse_sysctl_row {
+    my($row, $key, $major) = @_;
+    $major ||= do {
+        my %sw_vers = sw_vers();
+        (split m{[.]}xms, $sw_vers{ProductVersion} || q{})[0] || 0;
+    };
+    my $re_row = $major == 10 ? RE_SYSCTL_ROW : RE_OLD_SYSCTL_ROW;
+    my($name, $value) = split $re_row, $row, 2;
+    if ( ! $value && ( ! defined $value || $value ne '0' ) ) {
+        croak sprintf q(Can't happen: No value in output for property )
+                    . q('%s' inside row '%s' collected from key '%s'),
+                        $name || q([no name]),
+                        $row,
+                        $key;
+    }
+    return $name, $value;
+}
+
 sub _sysctl_not_exists {
     my($error) = @_;
     return if ! $error;
@@ -115,8 +141,8 @@ Sys::Info::Driver::OSX - OSX driver for Sys::Info
 
 =head1 DESCRIPTION
 
-This document describes version C<0.793> of C<Sys::Info::Driver::OSX>
-released on C<12 May 2011>.
+This document describes version C<0.794> of C<Sys::Info::Driver::OSX>
+released on C<16 May 2011>.
 
 This is the main module in the C<OSX> driver collection.
 
@@ -142,6 +168,10 @@ System call to system_profiler.
 
 System call to sw_vers.
 
+=head2 plist
+
+Converts a file or raw plist data into a Perl structure.
+
 =head1 AUTHOR
 
 Burak Gursoy <burak@cpan.org>.
